package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.common.utils.ResultUtil;
import com.huimv.guowei.admin.entity.BaseBuilding;
import com.huimv.guowei.admin.entity.dto.BasePigpenDto;
import com.huimv.guowei.admin.entity.vo.TreeBasePigpen;
import com.huimv.guowei.admin.mapper.BaseBuildingMapper;
import com.huimv.guowei.admin.service.IBaseBuildingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-13
 */
@Service
public class BaseBuildingServiceImpl extends ServiceImpl<BaseBuildingMapper, BaseBuilding> implements IBaseBuildingService {
    @Autowired
    private BaseBuildingMapper baseBuildingMapper;
//    @Autowired
//    private EnvDeviceMapper envDeviceMapper;

    @Override
    @Transactional
    public Result addPigpen(BasePigpenDto basePigpenDto) {
        Integer  num = baseBuildingMapper.selectCount(new QueryWrapper<BaseBuilding>().eq("build_name", basePigpenDto.getBuildName()));
        if (num  > 0){
            return ResultUtil.exist();
        }
        BaseBuilding baseBuilding = new BaseBuilding();
        BeanUtil.copyProperties(basePigpenDto, baseBuilding);
        baseBuilding.setParentId(0);
        baseBuilding.setFType(1);
        baseBuilding.setOther2("0");
        int insert = baseBuildingMapper.insert(baseBuilding);
        Integer id = baseBuilding.getId();
        String buildName = baseBuilding.getBuildName();
        Integer floorNum = basePigpenDto.getFloorNum();
        for (Integer integer = 1; integer <= floorNum; integer++) {
            BaseBuilding baseBuilding1 = new BaseBuilding();
            baseBuilding1.setFType(2);
            baseBuilding1.setParentId(id);
            baseBuilding1.setBuildName(buildName+Convert.numberToChinese(integer,false) +"层");
            baseBuilding1.setOther1(Convert.numberToChinese(integer,false) +"层");
            baseBuilding1.setOther2("0,"+id);
            baseBuilding1.setFarmId(baseBuilding.getFarmId());
            baseBuilding1.setStageCode(baseBuilding.getStageCode());
            baseBuildingMapper.insert(baseBuilding1);
        }
        return ResultUtil.addResult(insert);
    }

    @Override
    @Transactional
    public Result updatePigpen(BaseBuilding baseBuilding) {
        baseBuildingMapper.updateById(baseBuilding);
        Integer id = baseBuilding.getId();
        List<BaseBuilding> parentId = baseBuildingMapper.selectList(new QueryWrapper<BaseBuilding>().eq("parent_id", id));
        String buildName = baseBuilding.getBuildName();
        if (StringUtils.isNotBlank(buildName)){
            for (BaseBuilding pigpen : parentId) {
                String other1 = pigpen.getOther1();
                pigpen.setBuildName(buildName+other1);
                baseBuildingMapper.updateById(pigpen);
            }
        }
        return ResultUtil.updateResult(1);
    }

    @Override
    @Transactional
    public Result deletePigpen(Map<String, Integer> map) {
//        Integer integer = map.get("id");
//        Integer integer1 = envDeviceMapper.selectCount(new QueryWrapper<EnvDevice>().eq("unit_id", integer));
//        if (integer1 > 0){
//            return new Result(10001,"删除失败，该栋舍下有采集器",false);
//        }
//        List<BasePigpen> other2 = basePigpenMapper.selectList(new QueryWrapper<BasePigpen>().like("other2", integer));
//        for (BasePigpen basePigpen : other2) {
//            Integer count = envDeviceMapper.selectCount(new QueryWrapper<EnvDevice>().eq("unit_id", basePigpen.getId()));
//            if (count > 0){
//                return new Result(10001,"删除失败，该栋舍下有采集器",false);
//            }
//            basePigpenMapper.deleteById(basePigpen);
//        }
//        basePigpenMapper.deleteById(integer);
//        return ResultUtil.deleteResult(1);
        return null;
    }

    @Override
    public Result list(String farmCode, String buildName, String stageCode) {
        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(buildName),"build_name", buildName);
        queryWrapper.like(StringUtils.isNotBlank(stageCode),"stage_code", stageCode);
        queryWrapper.eq(StringUtils.isNotBlank(farmCode),"farm_id", farmCode);
        queryWrapper.orderByAsc("sort");
        //创建排序
        List<BaseBuilding> baseBuildings = baseBuildingMapper.selectList(queryWrapper);
        //将结果List改为树
        List<TreeBasePigpen> treeBasePigpens = parseBizBaseArea(baseBuildings);
        return new Result(ResultCode.SUCCESS,treeBasePigpens);
    }

    /**
     * 查询结果 转换成树形结构
     * @param bizBaseAreas 原始数据
     * @return 树
     */
    private List<TreeBasePigpen> parseBizBaseArea(List<BaseBuilding> bizBaseAreas){
        //构建需要展示的树形节点结构
        Map<String, TreeBasePigpen> nodeMap = bizBaseAreas.stream().map(baseBuilding -> {
            TreeBasePigpen  baseVo = new TreeBasePigpen();
            baseVo.setId(baseBuilding.getId()+"");
            baseVo.setFarmCode(baseBuilding.getFarmId());
            baseVo.setParentId(baseBuilding.getParentId());
            baseVo.setSort(baseBuilding.getSort());
            baseVo.setStageCode(baseBuilding.getStageCode());
            baseVo.setPigpenName(baseBuilding.getBuildName());
            baseVo.setType(baseBuilding.getFType());
            baseVo.setChildNode(new ArrayList<>());
            return baseVo;
        }).collect(Collectors.toMap(TreeBasePigpen::getId, b -> b,(k1, k2)->k1));
        //创建数组存父亲节点
        ArrayList<TreeBasePigpen> roots = new ArrayList<>();
        //构建树形
        nodeMap.values().forEach(item->{
            String parentCode = item.getParentId()+"";
            if (nodeMap.get(parentCode) == null){
                //父节点为空，说明当前节点就已经是父节点了 将该节点存起来
                roots.add(item);
            }else {
                //父节点 不为空，说明有父节点，拿到该节点的父节点的孩子节点（就是我自己),将该节点存起来
                nodeMap.get(parentCode).getChildNode().add(item);
            }
        });
        return roots;
    }
}
